package net.peachjean.modularwebapp;

import com.google.common.base.Optional;
import org.apache.commons.cli.*;

import java.io.File;
import java.io.PrintStream;

public class ModularWebappLauncher {

	public static void main(String[] args) throws Exception {
		new ModularWebappLauncher().run(args);
	}

	public void run(String... args) throws Exception {
		CommandLine commandLine = parseArgs(args);
		final ModularWebapp webapp = buildWebapp(commandLine);
		webapp.launchAndJoin();
	}

	private CommandLine parseArgs(final String... args) throws ParseException {
		CommandLineParser parser = new GnuParser();
		return parser.parse(buildOptions(), args);
	}

	private Options buildOptions() {
		Option port = new Option("p", "port", true, "The port to listen on. Defaults to 8080.");
		Option root = new Option("r", "root", true, "The root directory of the web container. " +
				"Defaults to the current working directory.");
		Option live = new Option("l", "live", false, "Is the application live? " +
				"This determines what stage to run Guice in.");
		Option dumpStructure = new Option("d", "dumpStructure", false, "If specified, the server structure will be " +
				"printed to stderr upon startup.");
		return new Options().addOption(port).addOption(root).addOption(live).addOption(dumpStructure);
	}

	private ModularWebapp buildWebapp(final CommandLine commandLine) throws Exception {
		int port = Integer.parseInt(commandLine.getOptionValue("p", "8080"));
		File root = new File(commandLine.getOptionValue("r", "."));
		boolean isLive = commandLine.hasOption("l");
		boolean dump = commandLine.hasOption("d");
		return createWebapp(port, root, isLive, dump);
	}

	protected ModularWebapp createWebapp(final int port, final File root, final boolean live, final boolean dump) {
		return new ModularWebapp(port, root, live, dump ? Optional.of(System.err) : Optional.<PrintStream>absent());
	}
}
